#include <stdio.h>
#include <stdlib.h>
#include <roxml.h>
#include "structures.h"
#include "competence.h"


/*!
* \file competence.c
* \brief Fichier contenant le code des fonctions liees aux arbres de comptences.
*/

/*! Copie de la struct Competence pour travailler

typedef struct Competence{
    char nomComp[TAILLE_NOMS];
    char id[TAILLE_ID];
    unsigned int prix_competence;
    struct Competence *frere;
    struct Competence *enfant;
    struct Effet_Competence *effet;
    boolean connu;
}Competence;

*/

/*!
 * \fn int init_Competence(Competence *noeud)
 * \brief La fonction initialise une competence.
 *
 * \return Un entier.
 */
Competence* init_Competence(Competence *noeud)
{
    noeud = malloc(sizeof(Competence));

    memset(noeud->nomComp, 0, sizeof(noeud->nomComp));
    memset(noeud->id, 0, sizeof(noeud->id));
    noeud->prix_competence = 0;
    noeud->frere = NULL;
    noeud->enfant = NULL;
    noeud->effet = malloc(sizeof(Effet_Competence));
    noeud->connu = 0;

    return noeud;
}

/* -------------------------------------------------------*/
/* Fonctions de test des competences */
/* -------------------------------------------------------*/

/*!
 * \fn int estVide(Competence *c)
 * \brief La fonction indique si une competence n'a pas d'enfants.
 *
 * \return vrai ou faux.
 */
int estVide(Competence *c)
{
    return(c->enfant==NULL);
}

/*!
 * \fn int aUnFrere(Competence *c)
 * \brief La fonction indique si une competence a un frre.
 *
 * \return vrai ou faux.
 */
int aUnFrere(Competence *c)
{
    return(c->frere!=NULL);
}

/* -------------------------------------------------------*/
/* Fonctions de set des competences */
/* -------------------------------------------------------*/

int set_comp_enfant(Competence *parent, Competence *nouvel_enfant)  /* Ne fonctionne pas */
{
    if(estVide(parent))
    {
        init_Competence(parent->enfant);
        parent->enfant = nouvel_enfant;
    }
    else  // Solution cracra mais a ne marche pas autrement (avec un while) //
    {
        set_comp_frere(parent->enfant, nouvel_enfant);
    }
    return 0;
}

int set_comp_frere(Competence *noeud, Competence *nouveau_frere)
{
    if(aUnFrere(noeud))
    {
        set_comp_frere(noeud->frere, nouveau_frere);
    }
    else
    {
        init_Competence(noeud->frere);
        noeud->frere = nouveau_frere;
    }
    return 0;
}

int set_comp_name(Competence *comp, char *nom_donne)
{
    strcpy(comp->nomComp, nom_donne);
    return 0;
}

int set_comp_id(Competence *comp, char *nom_donne)
{
    strcpy(comp->id, nom_donne);
    return 0;
}

int set_comp_prix(Competence *comp, int prix)
{
    comp->prix_competence=prix;
    return 0;
}

int set_comp_effet(Competence *comp, Effet_Competence *nouvel_effet)
{
    comp->effet = nouvel_effet;
    return 0;
}

/* -------------------------------------------------------*/
/* -------------------------------------------------------*/
/* -------------------------------------------------------*/


/*!
 * \fn int taille(Competence *noeud)
 * \brief La fonction renvoie le nombre d'enfants totaux d'une competence.
 *
 * \return Un entier positif.
 */
int taille(Competence *noeud)
{
    Competence *enfant;
    int compteur = 1;
    for (enfant = noeud->enfant; enfant != NULL; enfant = enfant->frere)
        compteur += taille(enfant);
    return compteur;
}

/*!
 * \fn unsigned int hauteur(Competence *noeud)
 * \brief La fonction indique la hauteur d'une competence dans l'arbre.
 *
 * \return Un entier positif.
 */
unsigned int hauteur(Competence *noeud)
{
    if (!estVide(noeud))
    {
        int compteur = 1;
        Competence *enfant;
        for (enfant = noeud->enfant; enfant != NULL; enfant = enfant->frere)
        {
            if (hauteur(enfant) > compteur)
                compteur = 1 + hauteur(enfant);
        }
        return compteur;
    }
    else
    {
        return 0;
    }
}

/*!
 * \fn boolean est_connue(Competence *c)
 * \brief La fonction indique si une competence est connue ou non.
 *
 * \return vrai ou faux.
 */
boolean est_connue(Competence *c)
{
    return (c->connu);
}


/*!
 * \fn unsigned int * NbEnfants(Competence *noeud)
 * \brief La fonction renvoie le nombre d'enfants d'une competence.
 *
 * \return Un entier positif.
 */
unsigned int NbEnfants(Competence *noeud)
{
    unsigned int nb;
    if (estVide(noeud))
    {
        nb = 0;
    }
    else
    {
        Competence *enfant;
        for(noeud->enfant; enfant!= NULL; enfant=enfant->frere);

        nb++;
    }
    return nb;
}

/*!
 * \fn boolean * est_complet(Competence *noeud)
 * \brief La fonction verifie si l'arbre est complet ou non.
 *
 * \return Un booleen.
 */
boolean est_complet(Competence *noeud)
{
    Competence *enfant;
    for(noeud->enfant; enfant!= NULL; enfant=enfant->frere)
    {
        if(!est_complet(enfant))
            return 0;
    }
    return 1;
}

int chemin_auto_competence(Competence* c)       /* Renvoie la liste des comptences non connues se trouvant sur le chemin d'une comptence slectionne */
{
    return 0;
}

Effet_Competence * get_effet(Competence *c)       /* Renvoie l'effet d'une competence */
{
    return c->effet;
}



Competence* Extraire_Arbres(char *filename)
{
    node_t *rootXML, *Arbres;
    int nb_Arbres=0, i_0=0;

    rootXML = roxml_load_doc(filename);
    Arbres = roxml_get_chld(rootXML,"Arbres",0);
    nb_Arbres = roxml_get_chld_nb(Arbres);
    printf("On a %d arbres de competences \n", nb_Arbres);

    /* ---------------------------------------------------------------------------------------------- */
    /* Extraction des arbres */
    /* ---------------------------------------------------------------------------------------------- */

    for(i_0=0; i_0<nb_Arbres; i_0++)
    {
        node_t *ArbreXML_Actuel = roxml_get_chld(Arbres, NULL, i_0);
        Competence *Arbre_Actuel;
        int nb_Comp1=0, i_1=0;

        Arbre_Actuel = init_Competence(Arbre_Actuel);

        nb_Comp1 = roxml_get_chld_nb(ArbreXML_Actuel);
        printf("L'arbre %d contient %d competences de niveau 1\n",i_0, nb_Comp1);
        set_comp_name(Arbre_Actuel,roxml_get_content(roxml_get_attr(ArbreXML_Actuel,"id",0),NULL,0,NULL));
        printf("\nL'arbre actuel a pour nom : %s\n", Arbre_Actuel->nomComp);

        /* ---------------------------------------------------------------------------------------------- */
        /* Extraction des competences 1 */
        /* ---------------------------------------------------------------------------------------------- */

        for(i_1=0; i_1<nb_Comp1; i_1++)
        {
            node_t *CompetenceXML_1;
            Competence *Competence_1;
            int nb_Comp2=0, i_2=0;

            Competence_1 = init_Competence(Competence_1);

            CompetenceXML_1 = roxml_get_chld(ArbreXML_Actuel, NULL, i_1);
            nb_Comp2 = roxml_get_chld_nb(CompetenceXML_1);
            printf("La competence numero %d de niveau 1 contient %d competences de niveau 2\n",i_1+1,nb_Comp2);

            set_comp_name(Competence_1,roxml_get_content(roxml_get_attr(CompetenceXML_1,"nom",0),NULL,0,NULL));
            set_comp_id(Competence_1,roxml_get_content(roxml_get_attr(CompetenceXML_1,"id",0),NULL,0,NULL));
            set_comp_prix(Competence_1,roxml_get_content(roxml_get_attr(CompetenceXML_1,"prix",0),NULL,0,NULL));

            printf("La competence_1 actuelle a pour nom : %s\n", Competence_1->nomComp);
            printf("La competence_1 actuelle a pour id : %s\n", Competence_1->id);
            printf("La competence_1 actuelle a pour prix : %s\n", Competence_1->prix_competence);


            for(i_2=0; i_2<nb_Comp2; i_2++)
            {
                node_t *CompetenceXML_2;
                Competence *Competence_2;
                int nb_Comp3=0, i_3=0;

                Competence_2 = init_Competence(Competence_2);
                CompetenceXML_2 = roxml_get_chld(CompetenceXML_1, NULL, i_2);
                nb_Comp3 = roxml_get_chld_nb(CompetenceXML_2);
                printf("La competence numero %d de niveau 2 contient %d competences de niveau 3\n",i_2+1,nb_Comp3);

                set_comp_name(Competence_2,roxml_get_content(roxml_get_attr(CompetenceXML_2,"nom",0),NULL,0,NULL));
                set_comp_id(Competence_2,roxml_get_content(roxml_get_attr(CompetenceXML_2,"id",0),NULL,0,NULL));
                set_comp_prix(Competence_2,roxml_get_content(roxml_get_attr(CompetenceXML_2,"prix",0),NULL,0,NULL));

                printf("La competence_2 actuelle a pour nom : %s\n", Competence_2->nomComp);
                printf("La competence_2 actuelle a pour id : %s\n", Competence_2->id);
                printf("La competence_2 actuelle a pour prix : %s\n", Competence_2->prix_competence);

                for(i_3=0; i_3<nb_Comp3; i_3++)
                {
                    node_t *CompetenceXML_3;
                    Competence *Competence_3;
                    int nb_Comp4=0, i_4=0;

                    Competence_3 = init_Competence(Competence_3);
                    CompetenceXML_3 = roxml_get_chld(CompetenceXML_2, NULL, i_3);
                    nb_Comp4 = roxml_get_chld_nb(CompetenceXML_3);
                    printf("La competence numero %d de niveau 3 contient %d competences de niveau 4\n",i_3+1,nb_Comp4);

                    set_comp_name(Competence_3,roxml_get_content(roxml_get_attr(CompetenceXML_3,"nom",0),NULL,0,NULL));
                    set_comp_id(Competence_3,roxml_get_content(roxml_get_attr(CompetenceXML_3,"id",0),NULL,0,NULL));
                    set_comp_prix(Competence_3,roxml_get_content(roxml_get_attr(CompetenceXML_3,"prix",0),NULL,0,NULL));

                    printf("La competence_3 actuelle a pour nom : %s\n", Competence_3->nomComp);
                    printf("La competence_3 actuelle a pour id : %s\n", Competence_3->id);
                    printf("La competence_3 actuelle a pour prix : %s\n", Competence_3->prix_competence);

                    for(i_4=0; i_4<nb_Comp4; i_4++)
                    {
                        node_t *CompetenceXML_4;
                        Competence *Competence_4;

                        Competence_4 = init_Competence(Competence_4);
                        CompetenceXML_4 = roxml_get_chld(CompetenceXML_3, NULL, i_4);

                        set_comp_name(Competence_4,roxml_get_content(roxml_get_attr(CompetenceXML_4,"nom",0),NULL,0,NULL));
                        set_comp_id(Competence_4,roxml_get_content(roxml_get_attr(CompetenceXML_4,"id",0),NULL,0,NULL));
                        set_comp_prix(Competence_4,roxml_get_content(roxml_get_attr(CompetenceXML_4,"prix",0),NULL,0,NULL));

                        printf("La competence_4 actuelle a pour nom : %s\n", Competence_4->nomComp);
                        printf("La competence_4 actuelle a pour id : %s\n", Competence_4->id);
                        printf("La competence_4 actuelle a pour prix : %s\n", Competence_4->prix_competence);

                        /* La competence 4 devient un enfant de la competence 3 */
                        set_comp_enfant(Competence_3, Competence_4);                         //FAIT PLANTER A PARTIR DE 3 NIVEAUX DE REMPLISSAGE
                    }
                    /* La competence 3 devient un enfant de la competence 2 */
                    //set_comp_enfant(Competence_2, Competence_3);
                }
                /* La competence 2 devient un enfant de la competence 1 */
                set_comp_enfant(Competence_1, Competence_2);
            }
            /* La competence 1 devient un enfant du premier noeud */
                //set_comp_enfant(Arbre_Actuel, Competence_1);
        }

    }



    return 0;
}



    int Test_Competences()
    {
        printf("Debut Test Competences\n");

        Competence *Comp_Test_Parent, *Comp_Test_Enfant, *Comp_Test_Petit_Enfant, *Comp_Test_Frere_Enfant;

        Comp_Test_Parent=init_Competence(Comp_Test_Parent);
        Comp_Test_Enfant=init_Competence(Comp_Test_Enfant);
        Comp_Test_Petit_Enfant=init_Competence(Comp_Test_Petit_Enfant);
        Comp_Test_Frere_Enfant=init_Competence(Comp_Test_Frere_Enfant);

        if(estVide(Comp_Test_Parent))   /* Test de estVide() */
        {
            printf("Parent n'a pas d'enfants\n");
        }

        printf("Le nom de parent est : %s\n", Comp_Test_Parent->nomComp);  /* Vaut NULL aprs l'init */

        set_comp_prix(Comp_Test_Parent, 4);
        set_comp_name(Comp_Test_Parent, "Pepe");
        set_comp_id(Comp_Test_Parent, "ID_PE");

        printf("Le nom de parent est : %s\n", Comp_Test_Parent->nomComp); /* Vaut bien le nom */

        set_comp_name(Comp_Test_Enfant, "Papa");
        set_comp_id(Comp_Test_Enfant, "ID_PA");
        set_comp_name(Comp_Test_Petit_Enfant, "Bebe");
        set_comp_id(Comp_Test_Enfant, "ID_BE");
        set_comp_name(Comp_Test_Frere_Enfant, "Tonton");
        set_comp_id(Comp_Test_Frere_Enfant, "ID_TON");

        /* Tests */
        printf("Le prix de Parent est %d\n", Comp_Test_Parent->prix_competence);
        printf("Le nom de Parent est %s\n", Comp_Test_Parent->nomComp);
        printf("L'ID de Parent est %s\n", Comp_Test_Parent->id);
        /* ---- */

        set_comp_enfant(Comp_Test_Parent, Comp_Test_Enfant); /* Association Parent-Enfant */

        set_comp_enfant(Comp_Test_Parent, Comp_Test_Frere_Enfant); /* Insertion d'un deuxime enfant (qui doit devenir le frre du premier) */

        printf("\nL'enfant de %s est %s\n", Comp_Test_Parent->nomComp, Comp_Test_Parent->enfant->nomComp);
        printf("L'autre enfant de %s est %s\n", Comp_Test_Parent->nomComp, Comp_Test_Parent->enfant->frere->nomComp);


        return 0;
}




